@@ -37,8 +37,8 @@ def satoh_solution(R, z, beta):
 
     Q = np.sqrt(b**2 + z**2)
     S = np.sqrt(a**2 + 2*a*Q + R**2 + z**2)
-    sig2z = G*M*Q*(a + 2*Q)/(3*Q*(a + 2*Q) + S**2)/(2*S)        # eq.(68)
-    v2phi = sig2z*(1 - 6*(R/S)**2)/(1 - beta) + G*M*R**2/S**3   # eq.(69)
+    sig2z = G*M*Q*(a + 2*Q)/(3*Q*(a + 2*Q) + S**2)/(2*S)        # C20 eq.(68)
+    v2phi = sig2z*(1 - 6*(R/S)**2)/(1 - beta) + G*M*R**2/S**3   # C20 eq.(69)
 
     return sig2z, v2phi
 
@@ -82,7 +82,7 @@ def jam_axi_intr_example():
 
     jam = jam_axi_intr(dens_lum, sigma_lum, qintr_lum, dens_pot, sigma_pot, qintr_pot,
                         mbh, Rbin, zbin, beta=beta, gamma=gamma, align='cyl',
-                        plot=1, data=data, errors=errors, ml=-1)
+                        plot=1, data=data, errors=errors, ml=None, nodots=1)
 
     print(f"M/L = {jam.ml:.3f}; chi2 = {jam.chi2:.3f}")
 
