@@ -23,7 +23,6 @@ V1.0.0: Michele Cappellari, Oxford, 08 November 2019
     - Written and tested.
 
 """
-import copy
 import numpy as np
 import matplotlib.pyplot as plt
 from scipy import ndimage
@@ -121,7 +120,7 @@ def intrinsic_moments_cyl(R, z,
     """
     # Make numerical quadrature insensitive to scaling
     scale = np.median(sigma_lum)
-    R, z, sigma_lum, sigma_pot = np.array([R, z, sigma_lum, sigma_pot])/scale
+    R, z, sigma_lum, sigma_pot = R/scale, z/scale, sigma_lum/scale, sigma_pot/scale
 
     args = [R, z, dens_lum, sigma_lum, qintr_lum, dens_pot, sigma_pot, qintr_pot, beta, gamma]
     nu = density(R, z, dens_lum, sigma_lum, qintr_lum).sum()
@@ -225,7 +224,7 @@ def intrinsic_moments_sph(R, z,
     
     # Make numerical quadrature insensitive to scaling
     scale = np.median(sigma_lum)
-    R, z, sigma_lum, sigma_pot = np.array([R, z, sigma_lum, sigma_pot])/scale
+    R, z, sigma_lum, sigma_pot = R/scale, z/scale, sigma_lum/scale, sigma_pot/scale
 
     r = np.sqrt(R**2 + z**2)
     th = np.arctan2(R, z)  # Angle from symmetry axis z
@@ -323,7 +322,6 @@ class mom_interp:
         self.sig2th = sig2th.reshape(nang, nrad)
         self.sig2phi = sig2phi.reshape(nang, nrad)
         self.v2phi = v2phi.reshape(nang, nrad)
-        self.nu = nu.reshape(nang, nrad)
         self.dens_lum = dens_lum
         self.sigma_lum = sigma_lum
         self.qintr_lum = qintr_lum
@@ -381,15 +379,17 @@ class jam_axi_intr:
 
         jam = jam_axi_intr(
             dens_lum, sigma_lum, qintr_lum, dens_pot, sigma_pot, qintr_pot,
-            mbh, Rbin, zbin, beta=None, gamma=None, epsrel=1e-2, quiet=False,
-            rbh=1, plot=True, nodots=True, fignum=1, data=None, errors=None,
-            ml=None, goodbins=None, nrad=20, nang=10, interp=True,
-            proj_cyl=False, align='cyl')
+            mbh, Rbin, zbin, align='cyl, ', beta=None, data=None,
+            epsrel=1e-2, errors=None, gamma=None, goodbins=None,
+            interp=True, ml=None, nang=10, nodots=True, nrad=20,
+            plot=True, proj_cyl=False, quiet=False, rbh=1)
 
         # The meaning of the output is different depending on `align`
         sig2R, sig2z, sig2phi, v2phi = jam.model  # with align='cyl'
         sig2r, sig2th, sig2phi, v2phi = jam.model  # with align='sph'
 
+        jam.plot()   # Generate data/model comparison
+
     Input Parameters
     ----------------
 
@@ -416,7 +416,7 @@ class jam_axi_intr:
     qintr_pot: array_like with shape (m,)
         vector containing the intrinsic axial ratio of the MGE
         Gaussians describing the galaxy total-mass density.
-    mbh: scalar
+    mbh: float
         Mass of a nuclear supermassive black hole in solar masses.
     Rbin: array_like with shape (p,)
         Vector with the ``R`` coordinates in ``pc`` of the bins (or pixels) at
@@ -434,6 +434,11 @@ class jam_axi_intr:
     Optional Keywords
     -----------------
 
+    align: {'cyl', 'sph'} optional
+        If ``align='cyl'`` the program computes the solution of the Jeans
+        equations with cylindrically-aligned velocity ellipsoid, presented
+        in `Cappellari (2008)`_. If ``align='sph'`` the spherically-aligned
+        solution of `Cappellari (2020)`_ is returned.
     beta: array_like with shape (n,)
         Vector with the axial anisotropy of the individual kinematic-tracer
         MGE Gaussians (Default: ``beta=np.zeros(n)``)::
@@ -441,7 +446,7 @@ class jam_axi_intr:
             beta = 1 - (sigma_th/sigma_r)^2  # with align=`sph`
             beta = 1 - (sigma_z/sigma_R)^2   # with align=`cyl`
 
-    data: array_like of shape (4, p)
+    data: array_like of shape (4, p), optional
         Four input vectors with the observed values of:
 
         - ``[sigR, sigz, sigphi, vrms_phi]`` in ``km/s``, when ``align='cyl'``
@@ -456,7 +461,10 @@ class jam_axi_intr:
         - ``[sigr, sigth, sigphi, vrms_phi]`` in ``km/s``, when ``align='sph'``,
           where ``vrms_phi`` is defined above.
 
-    errors: array_like of shape (4, p)
+    epsrel: float, optional
+        Relative error requested for the numerical quadratures, before
+        interpolation (Default: ``epsrel=1e-2``).
+    errors: array_like of shape (4, p), optional
         ``1sigma`` uncertainties on ``data``, in the same format (default 5 ``km/s``).
     gamma: array_like with shape (n,)
         Vector with the tangential anisotropy of the individual kinematic-tracer
@@ -465,10 +473,31 @@ class jam_axi_intr:
             gamma = 1 - (sigma_phi/sigma_r)^2  # with align=`sph`
             gamma = 1 - (sigma_phi/sigma_R)^2  # with align=`cyl`
 
-    goodbins: array_like with shape (4, p)
+    goodbins: array_like with shape (4, p), optional
         Boolean vector of the same shape as ``data`` with values ``True``
         for the bins which have to be included in the fit (if requested) and
         ``chi^2`` calculation (Default: fit all bins).
+    interp: bool, optional
+        If ``interp=False`` no interpolation is performed and the model is
+        computed at every set of input (R, z) coordinates.
+        If ``interp=True`` (default), the model is interpolated if the number
+        of requested input (R, z) coordinates is larger than ``nang*nrad``.
+    ml: float, optional
+        Mass-to-light ratio M/L. If ``ml=None`` (default) the M/L is fitted to
+        the data and the best-fitting value is returned in output.
+        The ``mbh`` is also scaled and becomes ``mbh*ml``.
+        If ``ml=1`` no scaling is applied to the model.
+    nang: int, optional
+        The number of linearly-spaced intervals in the eccentric anomaly at
+        which the model is evaluated before interpolation (default: ``nang=10``).
+    nrad: int, optional
+        The number of logarithmically spaced radial positions at which the
+        model is evaluated before interpolation. One may want to increase this
+        value if the model has to be evaluated over many orders of magnitude in
+        radius (default: ``nrad=20``).
+    plot: bool, optional
+        If ``plot=True`` (default) and ``data is not None``, produce a plot of
+        the data-model comparison at the end of the calculation.
     proj_cyl: bool, optional
         If ``align='sph'`` and ``proj_cyl=True``, the function projects the
         spherically-aligned moments to cylindrical coordinates and returns the
@@ -476,14 +505,17 @@ class jam_axi_intr:
         ``align='cyl'``. This is useful for a direct comparison of results with
         either the spherical or cylindrical alignment, as it allows one to fit
         the same data with both modelling assumptions.
+    quiet: bool, optional
+        If ``quiet=False`` (default), print the best-fitting M/L and chi2 at
+        the end for the calculation.
     rbh: float, optional
-        This scalar gives the sigma in arcsec of the Gaussian representing the
-        central black hole of mass ``mbh`` [See Section 3.1.2 of `Cappellari (2008)`_].
-        The gravitational potential is indistinguishable from a point source
-        for ``radii > 2*rbh``, so the default ``rbh=0.01`` arcsec is appropriate for
-        observations taken with most current telescopes.
-
-        ``rbh`` should not be decreased unless actually needed!
+        This scalar gives the sigma in pc of the Gaussian representing the
+        central black hole of mass ``mbh`` [See Section 3.1.2 of
+        `Cappellari (2008)`_]. The gravitational potential is indistinguishable
+        from a point source for ``radii > 2*rbh``, so the default ``rbh=1`` pc
+        is appropriate for observations taken with current telescopes.
+    kwargs: dict, optioonal
+        Additional keywords passed to ``plotbin.plot_velfield``.
 
     Output Parameters
     -----------------
@@ -526,17 +558,19 @@ class jam_axi_intr:
     .chi2: float
         Reduced chi^2 (chi^2/DOF) describing the quality of the fit::
 
-            chi2 = np.sum(((data[goodbins] - model[goodbins])/errors[goodbins])**2)
-                  / goodbins.sum()
+            d, m = (data/errors)[goodbins], (model/errors)[goodbins]
+            chi2 = ((d - m)**2).sum()/goodbins.sum()
 
     ###########################################################################
     """
     def __init__(self, dens_lum, sigma_lum, qintr_lum, dens_pot, sigma_pot, qintr_pot,
-                 mbh, Rbin, zbin, beta=None, gamma=None, epsrel=1e-2, quiet=False,
-                 rbh=1, plot=True, nodots=True, fignum=1, data=None, errors=None,
-                 ml=None, goodbins=None, nrad=20, nang=10, interp=True,
-                 proj_cyl=False, align='cyl'):
+                 mbh, Rbin, zbin, align='cyl, ', beta=None, data=None,
+                 epsrel=1e-2, errors=None, gamma=None, goodbins=None,
+                 interp=True, ml=None, nang=10, nrad=20, plot=True,
+                 proj_cyl=False, quiet=False, rbh=1, **kwargs):
 
+        assert align in ['sph', 'cyl'], "`align` must be 'sph' or 'cyl'"
+        assert (ml is None) or (ml > 0), "The input `ml` must be positive"
         if beta is None:
             beta = np.zeros_like(dens_lum)  # Anisotropy parameter beta = 1 - (sig_th/sig_r)**2  (beta=0-->circle)
         if gamma is None:
@@ -608,40 +642,45 @@ class jam_axi_intr:
             # Use full data for chi2 calculation
             model0 *= np.sqrt(ml)
             d, m, e, ok = map(np.ravel, [data, model0, errors, goodbins])
-            chi2 = np.sum(((d[ok] - m[ok])/e[ok])**2)/ok.sum()
+            chi2 = (((d[ok] - m[ok])/e[ok])**2).sum()/ok.sum()
 
-            if plot:
-                self.plot_jam(Rbin, zbin, data, model0, nu,
-                              nodots=nodots, fignum=fignum, align=align)
-
-        self.model = np.array(model)*ml   # model ~ V^2
+        self.Rbin = Rbin
+        self.zbin = zbin
+        self.align = align
+        self.model = np.array(model)*ml  # model ~ V^2
+        self.data = data
         self.flux = nu
         self.ml = ml
         self.chi2 = chi2
+        self.goodbins = goodbins
+
+        if plot and (data is not None):
+            self.plot(**kwargs)
 
 ##############################################################################
 
-    def plot_jam(self, Rbin, zbin, data, model, nu, nodots=True, fignum=1, align='cyl'):
+    def plot(self, **kwargs):
         """ Data-model comparison for jam_axi_intr """
 
-        fig, ax = plt.subplots(4, 2, sharex=True, sharey='col', num=fignum)
-        fig.subplots_adjust(hspace=0.03)
+        plt.figure('jam')
+        plt.clf()
+        fig, ax = plt.subplots(4, 2, sharex=True, sharey=True, num='jam', gridspec_kw={'hspace': 0})
 
-        if align == 'cyl':
+        if self.align == 'cyl':
             txt = [r"$\sigma_R$", r"$\sigma_z$", r"$\sigma_\phi$", r"$V^{\rm rms}_\phi$"]
         else:
             txt = [r"$\sigma_r$", r"$\sigma_\theta$", r"$\sigma_\phi$", r"$V^{\rm rms}_\phi$"]
 
         ax[0, 0].set_title("Data")
-        ax[0, 1].set_title(f"JAM$_{{\\rm {align}}}$ Model")
+        ax[0, 1].set_title(f"JAM$_{{\\rm {self.align}}}$ Model")
 
-        for j, (d, m, t) in enumerate(zip(data, model, txt)):
+        for j, (d, m, t) in enumerate(zip(self.data, np.sqrt(self.model.clip(0)), txt)):
             plt.sca(ax[j, 0])
             mn, mx = np.percentile(d, [1, 99])
-            plot_velfield(Rbin, zbin, d, nodots=nodots, vmin=mn, vmax=mx, flux=nu)
+            plot_velfield(self.Rbin, self.zbin, d, vmin=mn, vmax=mx, flux=self.flux, **kwargs)
 
             plt.sca(ax[j, 1])
-            plot_velfield(Rbin, zbin, m, nodots=nodots, vmin=mn, vmax=mx, flux=nu,
-                          colorbar=1, label=t)
+            plot_velfield(self.Rbin, self.zbin, m, vmin=mn, vmax=mx, flux=self.flux,
+                          colorbar=1, label=t, **kwargs)
 
 ##############################################################################
