@@ -702,11 +702,13 @@ class jam_axi_proj:
                 if ml is None:
                     ml = kappa = 1.0
                 else:
+                    kappa = 1.0
                     model *= np.sqrt(ml)
             else:
                 if kappa is None:
                     ml = kappa = 1.0
                 else:
+                    ml = 1.0
                     model *= kappa
 
         else:
